import { Host, h } from "@stencil/core";
import { ZFileUploadType } from "../../../beans";
export class ZFileUpload {
    constructor() {
        this.type = ZFileUploadType.DEFAULT;
        this.buttonVariant = undefined;
        this.acceptedFormat = undefined;
        this.fileMaxSize = undefined;
        this.mainTitle = undefined;
        this.description = undefined;
        this.uploadBtnLabel = "allega";
        this.dragAndDropLabel = "Rilascia i file in questa area per allegarli.";
        this.allowedFilesMessage = undefined;
        this.uploadClickableMessage = "Carica";
        this.uploadMessage = "o trascina dal tuo computer";
        this.errorModalTitle = "Errore di caricamento";
        this.errorModalMessage = undefined;
        this.uploadedFilesLabel = "File appena caricati";
        this.hasFileSection = true;
        this.inputName = "z-file-upload";
        this.showErrors = true;
        this._type = this.type;
        this.files = [];
        this.invalidFiles = new Map();
        this.input = undefined;
    }
    /** Listen `removeFile` event sent from z-file component */
    onFileRemoved(e) {
        this.removeFile(e.detail.fileName);
    }
    /** Listen fileDropped event sent from z-dragdrop-area component */
    fileDroppedListener(e) {
        this.input.files = e.detail;
        this.checkFilesValidity(this.input.files);
    }
    /** Get the list of uploaded files */
    async getFiles() {
        return this.files;
    }
    /** Remove a previously uploaded file */
    async removeFile(fileName) {
        const files = this.files;
        const index = files.findIndex((file) => file.name === fileName);
        if (index >= 0) {
            files.splice(index, 1);
            this.files = [...files];
        }
    }
    checkFilesValidity(files) {
        if (!files.length) {
            return;
        }
        Array.from(files).forEach((file) => {
            const fileSize = file.size / 1024 / 1024;
            const fileFormatOk = this.acceptedFormat
                .split(",")
                .some((ext) => file.name.toLowerCase().endsWith(ext.trim()));
            const fileSizeOk = fileSize <= this.fileMaxSize;
            if (fileSizeOk && fileFormatOk && !this.files.find((f) => f.name === file.name)) {
                this.files = [...this.files, file];
                this.fileInput.emit(file);
                this.input.value = "";
                return;
            }
            const sizeError = `Il file ${file.name} supera il limite di ${this.fileMaxSize}MB`;
            const formatError = `Il file ${file.name} ha un formato non supportato`;
            const errors = [
                !fileSizeOk && { cause: "size", message: sizeError },
                !fileFormatOk && { cause: "format", message: formatError },
            ].filter(Boolean);
            this.invalidFiles.set(file.name, errors);
            this.invalidFiles = new Map(this.invalidFiles); // trigger state update
            this.fileError.emit({ file: file.name, errors });
        });
    }
    resetErrors() {
        this.invalidFiles = new Map();
    }
    onFilesChange(e) {
        this.checkFilesValidity(e.target.files);
    }
    componentWillLoad() {
        // force default type on mobile and tablet viewport
        const mql = window.matchMedia("(max-width: 1151px)");
        this._type = mql.matches ? ZFileUploadType.DEFAULT : this.type;
        mql.addEventListener("change", (e) => {
            this._type = e.matches ? ZFileUploadType.DEFAULT : this.type;
        });
    }
    componentDidUpdate() {
        if (!this.showErrors || !this.invalidFiles.size) {
            return;
        }
        this.errorModal.open();
    }
    renderDescription(cssClass) {
        if (!this.description) {
            return;
        }
        return (h("span", { id: "description", class: cssClass }, this.description));
    }
    renderAllowedFilesMessage() {
        let fileFormatString = "";
        let fileWeightString = "";
        if (this.acceptedFormat) {
            const fileFormat = this.acceptedFormat
                .split(", ")
                .map((string) => string.substring(1).toUpperCase())
                .join(", ");
            fileFormatString = ` nei formati ${fileFormat}`;
        }
        if (this.fileMaxSize) {
            fileWeightString = ` per un massimo di ${this.fileMaxSize}MB di peso`;
        }
        const defaultMessage = fileFormatString || fileWeightString ? `Puoi allegare file${fileFormatString}${fileWeightString}.` : undefined;
        if (!this.allowedFilesMessage && !defaultMessage) {
            return;
        }
        return h("span", { class: "allowed-files-message body-3" }, this.allowedFilesMessage || defaultMessage);
    }
    renderFileSection() {
        if (!this.hasFileSection) {
            return;
        }
        return (h("section", { class: { "files-container": true, "hidden": !this.files.length } }, h("div", { class: "uploaded-files-label heading-3-sb" }, this.uploadedFilesLabel), h("div", { class: "files-wrapper" }, h("slot", { name: "files" }))));
    }
    renderInput() {
        return (h("input", { type: "file", name: this.inputName, multiple: true, onChange: (e) => this.onFilesChange(e), accept: this.acceptedFormat, ref: (val) => (this.input = val) }));
    }
    renderUploadButton() {
        return [
            h("z-button", { onClick: () => this.input.click(), onKeyPress: (e) => {
                    if (e.code == "Space" || e.code == "Enter") {
                        e.preventDefault();
                        this.input.click();
                    }
                }, id: "fileSelect", variant: this.buttonVariant, icon: "upload" }, this.uploadBtnLabel),
            this.renderInput(),
        ];
    }
    renderUploadLink() {
        return [
            this.renderInput(),
            h("span", { class: "body-1 upload-link-text" }, h("span", { tabIndex: 0, class: "body-1-sb upload-link", onClick: () => this.input.click(), onKeyPress: (e) => {
                    if (e.code == "Space" || e.code == "Enter") {
                        e.preventDefault();
                        this.input.click();
                    }
                } }, this.uploadClickableMessage), " ", this.uploadMessage),
        ];
    }
    renderDefaultMode() {
        return [
            this.renderDescription("body-3-sb"),
            this.renderAllowedFilesMessage(),
            this.renderFileSection(),
            this.renderUploadButton(),
        ];
    }
    renderDragDropMode() {
        return [
            this.renderFileSection(),
            h("z-dragdrop-area", { "drag-and-drop-label": this.dragAndDropLabel }, h("div", { class: "text-container" }, this.renderDescription("body-1"), this.renderUploadLink(), this.renderAllowedFilesMessage())),
        ];
    }
    renderFileErrors() {
        return Array.from(this.invalidFiles).map(([fileName, errors]) => {
            const prefix = `Il file ${fileName}`;
            return (h("span", { class: "error-message" }, "Il file ", h("span", { class: "body-3-sb" }, fileName), errors.map(({ message }) => message.replace(prefix, "")).join(" e "), "."));
        });
    }
    render() {
        return (h(Host, { key: '13ada780c37ca16c2153f8c1177864932fe299dd' }, this.mainTitle && (h("div", { key: 'dcf55ac93e6959136262d0422272ab4902c3cbed', id: "title", class: "heading-3-sb" }, this.mainTitle)), this._type == ZFileUploadType.DEFAULT ? this.renderDefaultMode() : this.renderDragDropMode(), !!this.invalidFiles.size && this.showErrors && (h("z-modal", { key: 'e7d8c001c513ab498779c752dc195ab03ec81a80', modalid: `file-upload-${this.type}-error-modal`, ref: (val) => (this.errorModal = val), modaltitle: this.errorModalTitle, onModalClose: () => this.resetErrors(), onModalBackgroundClick: () => this.resetErrors() }, h("div", { key: 'e5262f2f8b42dfe38ad4b794b08b4bf6b83b4a7a', slot: "modalContent" }, h("div", { key: '5e50b460de47813678ea1e2d3a2a3e0e76d0a6d0', class: "modal-wrapper body-3" }, this.errorModalMessage ? this.errorModalMessage : this.renderFileErrors()))))));
    }
    static get is() { return "z-file-upload"; }
    static get encapsulation() { return "scoped"; }
    static get originalStyleUrls() {
        return {
            "$": ["styles.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["styles.css"]
        };
    }
    static get properties() {
        return {
            "type": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ZFileUploadType",
                    "resolved": "ZFileUploadType.DEFAULT | ZFileUploadType.DRAGDROP",
                    "references": {
                        "ZFileUploadType": {
                            "location": "import",
                            "path": "../../../beans",
                            "id": "src/beans/index.tsx::ZFileUploadType"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Prop indicating the file upload type - can be default or dragdrop"
                },
                "attribute": "type",
                "reflect": true,
                "defaultValue": "ZFileUploadType.DEFAULT"
            },
            "buttonVariant": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ButtonVariant",
                    "resolved": "ButtonVariant.PRIMARY | ButtonVariant.SECONDARY | ButtonVariant.TERTIARY",
                    "references": {
                        "ButtonVariant": {
                            "location": "import",
                            "path": "../../../beans",
                            "id": "src/beans/index.tsx::ButtonVariant"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Prop indicating the button variant"
                },
                "attribute": "button-variant",
                "reflect": false
            },
            "acceptedFormat": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Prop indicating the accepted file type: ex \".pdf, .doc, .jpg\""
                },
                "attribute": "accepted-format",
                "reflect": false
            },
            "fileMaxSize": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Max file dimension in Megabyte"
                },
                "attribute": "file-max-size",
                "reflect": false
            },
            "mainTitle": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Title"
                },
                "attribute": "main-title",
                "reflect": false
            },
            "description": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Description"
                },
                "attribute": "description",
                "reflect": false
            },
            "uploadBtnLabel": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "upoload button label"
                },
                "attribute": "upload-btn-label",
                "reflect": false,
                "defaultValue": "\"allega\""
            },
            "dragAndDropLabel": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "drag & drop button label"
                },
                "attribute": "drag-and-drop-label",
                "reflect": false,
                "defaultValue": "\"Rilascia i file in questa area per allegarli.\""
            },
            "allowedFilesMessage": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "allowed file message"
                },
                "attribute": "allowed-files-message",
                "reflect": false
            },
            "uploadClickableMessage": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "upload clickable message"
                },
                "attribute": "upload-clickable-message",
                "reflect": false,
                "defaultValue": "\"Carica\""
            },
            "uploadMessage": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "upload message"
                },
                "attribute": "upload-message",
                "reflect": false,
                "defaultValue": "\"o trascina dal tuo computer\""
            },
            "errorModalTitle": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "error modal title"
                },
                "attribute": "error-modal-title",
                "reflect": false,
                "defaultValue": "\"Errore di caricamento\""
            },
            "errorModalMessage": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "error modal message"
                },
                "attribute": "error-modal-message",
                "reflect": false
            },
            "uploadedFilesLabel": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "loaded files label"
                },
                "attribute": "uploaded-files-label",
                "reflect": false,
                "defaultValue": "\"File appena caricati\""
            },
            "hasFileSection": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "uploaded files history rendering"
                },
                "attribute": "has-file-section",
                "reflect": false,
                "defaultValue": "true"
            },
            "inputName": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Value to set on the file input's `name` attribute (for use with forms)"
                },
                "attribute": "input-name",
                "reflect": false,
                "defaultValue": "\"z-file-upload\""
            },
            "showErrors": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Whether to show errors in the internal modal or leave the handling to the app.\nErrors will still be emitted."
                },
                "attribute": "show-errors",
                "reflect": false,
                "defaultValue": "true"
            }
        };
    }
    static get states() {
        return {
            "_type": {},
            "files": {},
            "invalidFiles": {},
            "input": {}
        };
    }
    static get events() {
        return [{
                "method": "fileInput",
                "name": "fileInput",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Emitted when user select one or more files"
                },
                "complexType": {
                    "original": "File",
                    "resolved": "File",
                    "references": {
                        "File": {
                            "location": "global",
                            "id": "global::File"
                        }
                    }
                }
            }, {
                "method": "fileError",
                "name": "fileError",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Emits an array of error messages related to one or more files not allowed.\nEmitted when some file is dropped or selected."
                },
                "complexType": {
                    "original": "{file: string; errors: ZFileUploadError[]}",
                    "resolved": "{ file: string; errors: ZFileUploadError[]; }",
                    "references": {
                        "ZFileUploadError": {
                            "location": "local",
                            "path": "/home/runner/work/design-system/design-system/src/components/file-upload/z-file-upload/index.tsx",
                            "id": "src/components/file-upload/z-file-upload/index.tsx::ZFileUploadError"
                        }
                    }
                }
            }];
    }
    static get methods() {
        return {
            "getFiles": {
                "complexType": {
                    "signature": "() => Promise<File[]>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        },
                        "File": {
                            "location": "global",
                            "id": "global::File"
                        }
                    },
                    "return": "Promise<File[]>"
                },
                "docs": {
                    "text": "Get the list of uploaded files",
                    "tags": []
                }
            },
            "removeFile": {
                "complexType": {
                    "signature": "(fileName: string) => Promise<void>",
                    "parameters": [{
                            "name": "fileName",
                            "type": "string",
                            "docs": ""
                        }],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "Remove a previously uploaded file",
                    "tags": []
                }
            }
        };
    }
    static get elementRef() { return "host"; }
    static get listeners() {
        return [{
                "name": "removeFile",
                "method": "onFileRemoved",
                "target": undefined,
                "capture": false,
                "passive": false
            }, {
                "name": "fileDropped",
                "method": "fileDroppedListener",
                "target": undefined,
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=index.js.map
