import { Fragment, Host, h } from "@stencil/core";
import { ButtonVariant, ControlSize, KeyboardCode, OffCanvasVariant, TransitionDirection } from "../../beans";
import { Breakpoints } from "../../constants/breakpoints";
import { containsElement } from "../../utils/utils";
const SUPPORT_INTERSECTION_OBSERVER = typeof IntersectionObserver !== "undefined";
/**
 * @slot title - Slot for the main title
 * @slot top-subtitle - Slot for the top subtitle. It will not appear in stuck header.
 * @slot stucked-title - Title for the stuck header. By default it uses the text from the `title` slot.
 * @slot product-logo - To insert the product logo, it should be used with an img tag.
 * @cssprop --app-header-content-max-width - Use it to set header's content max width. Useful when the project use a fixed width layout. Defaults to `100%`.
 * @cssprop --app-header-top-offset - Top offset for the stuck header. Useful when there are other fixed elements above the header. Defaults to `48px` (the height of the main topbar).
 * @cssprop --app-header-bg - Header background color. Defaults to `--color-surface01`.
 * @cssprop --app-header-text-color - Header text color. Defaults to `--color-default-text`.
 * @cssprop --app-header-stucked-bg - Stuck header background color. Defaults to `--color-surface01`.
 * @cssprop --app-header-stucked-text-color - Stuck header text color. Defaults to `--color-default-text`.
 */
export class ZAppHeader {
    onStuck() {
        const scrollParent = this.scrollParent;
        if (!scrollParent) {
            return;
        }
        this.sticking.emit(this._stuck);
    }
    setMenuFloatingMode() {
        if (this.menuElements.length === 0) {
            return;
        }
        this.menuElements.forEach((element) => {
            element.open = false;
            element.verticalContext = this.drawerOpen;
        });
    }
    onStuckChange() {
        var _a, _b;
        if (!this.container) {
            return;
        }
        if (this.stuck) {
            (_a = this.stuckIntersecObserver) === null || _a === void 0 ? void 0 : _a.observe(this.container);
        }
        else {
            this._stuck = false;
            (_b = this.stuckIntersecObserver) === null || _b === void 0 ? void 0 : _b.unobserve(this.container);
        }
    }
    get title() {
        var _a;
        return (_a = this.hostElement.querySelector('[slot="title"]')) === null || _a === void 0 ? void 0 : _a.textContent.trim();
    }
    get scrollParent() {
        const parent = this.hostElement.offsetParent;
        if (parent === document.body || parent === document.documentElement) {
            return window;
        }
        return parent;
    }
    get canShowMenu() {
        return !this.enableOffcanvas && this.menuElements.length > 0 && !this.isMobile && !this.drawerOpen;
    }
    get focusableMenu() {
        return this.menuElements.find((el) => el.htmlTabindex === 0);
    }
    openDrawer() {
        this.drawerOpen = true;
        this.menuElements.forEach((element, index) => (element.htmlTabindex = index === 0 ? 0 : -1));
        setTimeout(() => this.menuElements[0].setFocus(), 400); /* wait for the 400ms offcanvas transition */
    }
    closeDrawer() {
        this.drawerOpen = false;
        setTimeout(() => this.burgerButton.focus(), 100);
    }
    collectMenuElements() {
        this.menuElements = Array.from(this.hostElement.querySelectorAll('[slot="menu"]'));
        this.menuElements.forEach((element, index) => (element.htmlTabindex = index === 0 ? 0 : -1));
        this.menuLength = this.menuElements.length;
        if (!this.enableOffcanvas) {
            this.menuWidth =
                this.menuElements.reduce((acc, el) => acc + el.getBoundingClientRect().width, 0) +
                    (this.menuLength - 1) * 32 /* 32px is the gap between each menu item */;
        }
        this.setMenuFloatingMode();
    }
    /** Automatically use offcanvas mode when the menubar doesn't fit in the header. */
    setupMenuResizeObserver() {
        if (this.enableOffcanvas) {
            return;
        }
        this.menuResizeObserver = new ResizeObserver((entries) => {
            if (this.isMobile) {
                this.enableOffcanvas = true;
                return;
            }
            const containerWidth = entries[0].contentBoxSize[0].inlineSize;
            if (this.menuWidth === 0) {
                return;
            }
            if (this.menuWidth > containerWidth && !this.enableOffcanvas) {
                this.enableOffcanvas = true;
            }
            else if (this.menuWidth <= containerWidth && this.enableOffcanvas) {
                this.enableOffcanvas = false;
            }
        });
        this.menuResizeObserver.observe(this.container);
    }
    hasSlot(slotName) {
        return this.hostElement.querySelector(`[slot="${slotName}"]`) !== null;
    }
    moveFocus(current, receiver) {
        current.htmlTabindex = -1;
        receiver.setFocus();
    }
    onOffcanvasKeydown(ev) {
        var _a, _b;
        if (ev.key !== KeyboardCode.TAB || !this.drawerOpen) {
            return;
        }
        const closestMenu = ev.target.closest("z-menu");
        if (closestMenu) {
            ev.preventDefault();
            ev.stopPropagation();
            // restore tabindex to the zmenu containing the focused element
            closestMenu.htmlTabindex = 0;
            this.closeDrawerButton.focus();
        }
        else if (ev.target === this.closeDrawerButton) {
            ev.preventDefault();
            ev.stopPropagation();
            if (ev.shiftKey) {
                // give focus to the last open menu if any or the last menu otherwhise
                ((_a = this.menuElements.filter((menu) => menu.open).pop()) !== null && _a !== void 0 ? _a : this.menuElements[this.menuLength - 1]).setFocus();
            }
            else {
                // give focus to the first open menu if any or the first menu otherwhise
                ((_b = this.menuElements.find((menu) => menu.open)) !== null && _b !== void 0 ? _b : this.menuElements[0]).setFocus();
            }
        }
    }
    /** Close each menu except the one receiving focus/click, if any */
    manageMenus(ev) {
        const targetMenu = this.menuElements.find((menu) => containsElement(menu, ev.target));
        if (targetMenu) {
            this.menuElements.forEach((menu) => {
                if (menu === targetMenu) {
                    return;
                }
                menu.htmlTabindex = -1;
                if (!this.drawerOpen) {
                    menu.open = false;
                }
            });
        }
        else if (!this.drawerOpen) {
            this.menuElements.forEach((menu) => {
                menu.open = false;
            });
        }
    }
    handleKeydown(ev) {
        var _a, _b;
        if (ev.key === KeyboardCode.ESC && this.drawerOpen) {
            this.closeDrawer();
            return;
        }
        if (!this.menuElements.some((elem) => elem.contains(ev.target))) {
            return;
        }
        const current = this.focusableMenu;
        const currentIndex = this.menuElements.indexOf(current);
        let receiver;
        if ((ev.key === KeyboardCode.ARROW_RIGHT && !current.verticalContext) ||
            (ev.key === KeyboardCode.ARROW_DOWN && current.verticalContext)) {
            receiver = (_a = this.menuElements[currentIndex + 1]) !== null && _a !== void 0 ? _a : this.menuElements[0];
        }
        else if ((ev.key === KeyboardCode.ARROW_LEFT && !current.verticalContext) ||
            (ev.key === KeyboardCode.ARROW_UP && current.verticalContext)) {
            receiver = (_b = this.menuElements[currentIndex - 1]) !== null && _b !== void 0 ? _b : this.menuElements[this.menuLength - 1];
        }
        if (!receiver) {
            return;
        }
        if (ev.key === KeyboardCode.ARROW_UP && current.verticalContext) {
            if (receiver.open) {
                current.htmlTabindex = -1;
                receiver.focusLastItem();
                return;
            }
        }
        this.moveFocus(current, receiver);
    }
    renderSeachbar() {
        if (this.isMobile && !this.searchPageUrl && this._stuck) {
            return;
        }
        if (this.searchPageUrl && (this.isMobile || this.isTablet)) {
            return (h("z-button", { class: "search-page-button", variant: ButtonVariant.SECONDARY, href: this.searchPageUrl, icon: "search", size: ControlSize.X_SMALL }));
        }
        return (h("z-searchbar", { value: this.searchString, placeholder: this.searchPlaceholder, showSearchButton: true, searchButtonIconOnly: this.isMobile || this.isTablet, size: ControlSize.X_SMALL, variant: ButtonVariant.SECONDARY, preventSubmit: this.searchString.length < 3, onSearchTyping: (e) => (this.searchString = e.detail) }));
    }
    renderProductLogos() {
        return (h(Fragment, null, this.enableZLogo && (h("span", { class: "z-logo" }, h("img", { alt: "Logo Zanichelli" }))), this.hostElement.querySelector("[slot='product-logo']") && (h("span", { class: "product-logo" }, h("slot", { name: "product-logo" })))));
    }
    renderMenuButton() {
        return (this.menuLength > 0 &&
            (this.enableOffcanvas || this._stuck || this.isMobile) && (h("button", { ref: (el) => (this.burgerButton = el), class: "drawer-trigger", "aria-label": "Apri menu", "aria-haspopup": "menu", "aria-expanded": `${this.drawerOpen}`, "aria-controls": "offcanvas-menu", onClick: this.openDrawer }, h("z-icon", { name: "burger-menu" }))));
    }
    renderOffcanvas() {
        return (h("z-offcanvas", { id: "offcanvas-menu", variant: OffCanvasVariant.OVERLAY, transitiondirection: TransitionDirection.RIGHT, open: this.drawerOpen, onCanvasOpenStatusChanged: (ev) => (this.drawerOpen = ev.detail), onKeyDown: this.onOffcanvasKeydown }, h("div", { slot: "canvasContent" }, h("button", { ref: (el) => (this.closeDrawerButton = el), class: "drawer-close", "aria-label": "Chiudi menu", onClick: this.closeDrawer, "aria-hidden": `${!this.drawerOpen}`, disabled: !this.drawerOpen }, h("z-icon", { name: "close" })), h("div", { class: "drawer-content", "aria-hidden": `${!this.drawerOpen}` }, h("slot", { name: "menu", onSlotchange: this.collectMenuElements })))));
    }
    renderStuck() {
        return (h("div", { class: "heading-stuck" }, h("div", { class: "heading-stuck-content" }, this.renderMenuButton(), h("div", { class: "heading-title" }, this.renderProductLogos(), h("div", { class: "stucked-title" }, h("slot", { name: "stucked-title" }, this.title))), this.enableSearch && this.renderSeachbar())));
    }
    constructor() {
        this.menuElements = [];
        this.stuckIntersecObserver = SUPPORT_INTERSECTION_OBSERVER
            ? new IntersectionObserver(([entry]) => {
                this._stuck = !entry.isIntersecting;
            }, { threshold: 0.5 })
            : undefined;
        this.stuck = false;
        this.enableOffcanvas = false;
        this.enableSearch = false;
        this.searchPlaceholder = "Cerca";
        this.searchString = "";
        this.searchPageUrl = undefined;
        this.enableZLogo = true;
        this.drawerOpen = false;
        this._stuck = false;
        this.menuLength = undefined;
        this.isMobile = true;
        this.isTablet = false;
        this.openDrawer = this.openDrawer.bind(this);
        this.closeDrawer = this.closeDrawer.bind(this);
        this.collectMenuElements = this.collectMenuElements.bind(this);
        this.onOffcanvasKeydown = this.onOffcanvasKeydown.bind(this);
    }
    componentWillLoad() {
        this.collectMenuElements();
        // mobile and tablet media queries listeners
        const mobileMediaQuery = window.matchMedia(`(max-width: ${Breakpoints.MOBILE}px)`);
        this.isMobile = mobileMediaQuery.matches;
        mobileMediaQuery.addEventListener("change", (e) => (this.isMobile = e.matches));
        const tabletMediaQuery = window.matchMedia(`(min-width: ${Breakpoints.MOBILE + 1}px) and (max-width: ${Breakpoints.TABLET}px)`);
        this.isTablet = tabletMediaQuery.matches;
        tabletMediaQuery.addEventListener("change", (e) => (this.isTablet = e.matches));
    }
    componentDidLoad() {
        this.onStuckChange();
        this.setupMenuResizeObserver();
    }
    disconnectedCallback() {
        var _a;
        (_a = this.menuResizeObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
    }
    render() {
        const hasTopSubtitle = this.hasSlot("top-subtitle");
        return (h(Host, { key: '607b7e24e9f285a68b2b060594d7f05e1858d5cb', "menu-length": this.menuLength }, h("div", { key: 'de8f54eeebf6566e213aa26f938de37837a4eaee', class: { "heading-panel": true, "has-menubar": this.menuLength > 0 && !this.enableOffcanvas }, ref: (el) => (this.container = el) }, h("div", { key: 'f11b563767f199727353f296321d0668f8444465', class: "heading-container" }, ((!this.enableSearch && this.isMobile) || !this.isMobile) && (h("div", { key: '19345fabd859f0d6bad18ee1cdb1c03d959f888a', class: "top-subtitle" }, h("slot", { key: '47a905ab7dc9c88c99d296bc57021bb15ce5ca12', name: "top-subtitle" }))), h("div", { key: 'c2143a9324af347082cdd40584622818f631cea3', class: "heading-title" }, this.renderMenuButton(), !hasTopSubtitle && !this._stuck && this.renderProductLogos(), h("slot", { key: '90ebc7860504f24edf3b4380d716eedf4674ccab', name: "title" }), this.enableSearch && !this.isMobile && this.renderSeachbar()), this.enableSearch && this.isMobile && this.renderSeachbar()), h("nav", { key: '43fb23d049539010c329b0b4c03df3651ad0962b', class: "menu-container", "aria-label": this.title || undefined }, this.canShowMenu && (h("div", { key: 'e08c4cd11e0fb472ae66ed70c7129977175eaa1c', role: "menubar", "aria-label": this.title || undefined }, h("slot", { key: '5daa80b2f4cd1274f2cf564d29677a2c426a4359', name: "menu", onSlotchange: this.collectMenuElements }))))), this.renderOffcanvas(), this._stuck && this.renderStuck()));
    }
    static get is() { return "z-app-header"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["styles.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["styles.css"]
        };
    }
    static get properties() {
        return {
            "stuck": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Stuck mode for the header.\nYou can programmatically set it using an IntersectionObserver."
                },
                "attribute": "stuck",
                "reflect": true,
                "defaultValue": "false"
            },
            "enableOffcanvas": {
                "type": "boolean",
                "mutable": true,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "When enabled, the menu bar is not displayed and a burger icon appears to open the offcanvas menu. Automatically enabled on mobile and when the menu items overflow the container."
                },
                "attribute": "enable-offcanvas",
                "reflect": true,
                "defaultValue": "false"
            },
            "enableSearch": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Enable the search bar."
                },
                "attribute": "enable-search",
                "reflect": true,
                "defaultValue": "false"
            },
            "searchPlaceholder": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Placeholder text for the search bar."
                },
                "attribute": "search-placeholder",
                "reflect": false,
                "defaultValue": "\"Cerca\""
            },
            "searchString": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Search string for the search bar."
                },
                "attribute": "search-string",
                "reflect": false,
                "defaultValue": "\"\""
            },
            "searchPageUrl": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Url to the search page.\nSet this prop and `enableSearch` to show a link-button on mobile and tablet viewports, instead of the normal searchbar.\nThe link will also appear on the sticky header."
                },
                "attribute": "search-page-url",
                "reflect": false
            },
            "enableZLogo": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Enable laZ logo."
                },
                "attribute": "enable-z-logo",
                "reflect": true,
                "defaultValue": "true"
            },
            "drawerOpen": {
                "type": "boolean",
                "mutable": true,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The opening state of the drawer."
                },
                "attribute": "drawer-open",
                "reflect": false,
                "defaultValue": "false"
            }
        };
    }
    static get states() {
        return {
            "_stuck": {},
            "menuLength": {},
            "isMobile": {},
            "isTablet": {}
        };
    }
    static get events() {
        return [{
                "method": "sticking",
                "name": "sticking",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Emitted when the `stuck` state of the header changes"
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
    static get elementRef() { return "hostElement"; }
    static get watchers() {
        return [{
                "propName": "_stuck",
                "methodName": "onStuck"
            }, {
                "propName": "drawerOpen",
                "methodName": "setMenuFloatingMode"
            }, {
                "propName": "stuck",
                "methodName": "onStuckChange"
            }];
    }
    static get listeners() {
        return [{
                "name": "focusin",
                "method": "manageMenus",
                "target": "document",
                "capture": false,
                "passive": true
            }, {
                "name": "click",
                "method": "manageMenus",
                "target": "document",
                "capture": false,
                "passive": true
            }, {
                "name": "keydown",
                "method": "handleKeydown",
                "target": undefined,
                "capture": false,
                "passive": true
            }];
    }
}
//# sourceMappingURL=index.js.map
