import { Host, h } from "@stencil/core";
import { KeyboardCode } from "../../beans";
/**
 * A component to create submenus inside the ZMenu.
 * @slot - Label of the menu section.
 * @slot section - Single entry of the menu section. Set the same slot name to different elements to put many of them. Add the `data-active` attribute to a slotted element to highlight it (`active` also works but `data-active` is preferable).
 */
export class ZMenuSection {
    get focusableItem() {
        return this.items.find(({ tabIndex }) => tabIndex === 0);
    }
    toggle() {
        if (!this.hasItems) {
            return;
        }
        this.open = !this.open;
    }
    setItemsA11yAttrs() {
        if (!this.hasItems) {
            return;
        }
        this.items.forEach((item) => {
            item.setAttribute("role", "menuitem");
            item.tabIndex = -1;
        });
    }
    onItemsChange() {
        this.items = Array.from(this.host.querySelectorAll('[slot="section"]'));
        this.hasItems = this.items.length > 0;
        this.setItemsA11yAttrs();
    }
    /**
     * Sets slotted item text as `data-text` attribute value, to let CSS use it through `attr()`.
     * @param ev Slotchange event
     */
    onLabelSlotChange(ev) {
        const labelElement = ev.target.assignedElements()[0];
        labelElement.dataset.text = labelElement === null || labelElement === void 0 ? void 0 : labelElement.textContent;
    }
    onLabelClick() {
        this.toggle();
        this.setFocus();
    }
    onLabelKeydown(ev) {
        if (ev.key === KeyboardCode.ENTER || ev.key === KeyboardCode.SPACE) {
            ev.preventDefault();
            ev.stopPropagation();
            this.toggle();
            if (this.open) {
                this.focustFirstItem();
            }
        }
    }
    /**
     * Move focus and adjust the tabindex value of `receiver` and `current` elements,
     * setting -1 to the `current` and 0 to the `receiver`, then focus the `receiver` element.
     */
    moveFocus(receiver, current) {
        receiver.tabIndex = 0;
        setTimeout(() => {
            receiver.focus();
        }, 100);
        if (current) {
            current.tabIndex = -1;
        }
    }
    focustFirstItem() {
        this.moveFocus(this.items[0]);
    }
    /** Focus the last item. */
    async focusLastItem() {
        this.moveFocus(this.items[this.items.length - 1]);
    }
    /** Set tabindex of the label to 0, then focus it. */
    async setFocus() {
        this.htmlTabindex = 0;
        setTimeout(() => {
            this.label.focus();
        }, 100);
    }
    onOpenChange() {
        if (!this.open) {
            this.setItemsA11yAttrs();
            this.closed.emit();
        }
        else {
            this.opened.emit();
        }
    }
    onTabindexChange() {
        if (!this.label) {
            return;
        }
        this.label.tabIndex = this.htmlTabindex;
    }
    onItemsKeydown(ev) {
        switch (ev.key) {
            case KeyboardCode.ESC:
            case KeyboardCode.ARROW_LEFT:
                if (!this.open) {
                    break;
                }
                ev.preventDefault();
                ev.stopPropagation();
                this.moveFocus(this.label, this.focusableItem);
                this.open = false;
                break;
            case KeyboardCode.ARROW_RIGHT:
                if (this.open) {
                    break;
                }
                ev.preventDefault();
                ev.stopPropagation();
                this.open = true;
                this.focustFirstItem();
                break;
            case KeyboardCode.ARROW_DOWN: {
                if (!this.open) {
                    break;
                }
                if (document.activeElement === this.host) {
                    ev.preventDefault();
                    ev.stopPropagation();
                    this.focustFirstItem();
                    break;
                }
                const currentIndex = this.items.indexOf(this.focusableItem);
                const receiver = this.items[currentIndex + 1];
                if (receiver) {
                    ev.preventDefault();
                    ev.stopPropagation();
                    this.moveFocus(receiver, this.focusableItem);
                }
                else {
                    this.htmlTabindex = 0;
                    this.focusableItem.tabIndex = -1;
                }
                break;
            }
            case KeyboardCode.ARROW_UP: {
                if (!this.open || document.activeElement === this.host) {
                    break;
                }
                const currentIndex = this.items.indexOf(this.focusableItem);
                const receiver = this.items[currentIndex - 1];
                ev.preventDefault();
                ev.stopPropagation();
                if (receiver) {
                    this.moveFocus(receiver, this.focusableItem);
                }
                else {
                    // since there isn't a previous item to focus, give the focus to the label element
                    this.focusableItem.tabIndex = -1;
                    this.setFocus();
                }
                break;
            }
        }
    }
    constructor() {
        this.active = undefined;
        this.htmlTabindex = -1;
        this.open = undefined;
        this.hasItems = undefined;
        this.items = undefined;
        this.toggle = this.toggle.bind(this);
        this.onItemsChange = this.onItemsChange.bind(this);
        this.onLabelSlotChange = this.onLabelSlotChange.bind(this);
        this.onLabelClick = this.onLabelClick.bind(this);
        this.onLabelKeydown = this.onLabelKeydown.bind(this);
        this.onItemsKeydown = this.onItemsKeydown.bind(this);
    }
    connectedCallback() {
        this.onItemsChange();
    }
    render() {
        return (h(Host, { key: 'b308fce3890a3094dca7d5de4f5837b47d6f2796', open: this.open }, h("button", { key: 'dd6eaa241e2f26c79b7fa0e6f7e14049c20b3ed6', ref: (el) => (this.label = el), class: "label", "aria-haspopup": `${!!this.hasItems}`, "aria-expanded": `${!!this.open}`, role: "menuitem", tabindex: this.htmlTabindex, onClick: this.onLabelClick, onKeyDown: this.onLabelKeydown }, h("slot", { key: '74f5c61fcf32df69b0b5d17ff0f06b5b62f940fa', onSlotchange: this.onLabelSlotChange }), this.hasItems && h("z-icon", { key: '8338d8b57893c110475fe980320310236abb3446', name: this.open ? "chevron-up" : "chevron-down" })), h("div", { key: '4df11f5eaa5226c24930527caba7f0bfa048946a', class: "items", role: "menu", hidden: !this.open }, h("slot", { key: '8393b22b945ec02e32e78248090505a012529253', name: "section", onSlotchange: this.onItemsChange }))));
    }
    static get is() { return "z-menu-section"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["styles.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["styles.css"]
        };
    }
    static get properties() {
        return {
            "active": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Active state"
                },
                "attribute": "active",
                "reflect": true
            },
            "htmlTabindex": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Tabindex value to set on the label button.\nUseful to manage keyboard navigation focus with roving tabindex handled by this component's parent (usually ZMenu)."
                },
                "attribute": "html-tabindex",
                "reflect": false,
                "defaultValue": "-1"
            },
            "open": {
                "type": "boolean",
                "mutable": true,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The opening state of the section."
                },
                "attribute": "open",
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "hasItems": {},
            "items": {}
        };
    }
    static get events() {
        return [{
                "method": "opened",
                "name": "opened",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "The section has been opened."
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }, {
                "method": "closed",
                "name": "closed",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "The section has been closed."
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
    static get methods() {
        return {
            "focusLastItem": {
                "complexType": {
                    "signature": "() => Promise<void>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "Focus the last item.",
                    "tags": []
                }
            },
            "setFocus": {
                "complexType": {
                    "signature": "() => Promise<void>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "Set tabindex of the label to 0, then focus it.",
                    "tags": []
                }
            }
        };
    }
    static get elementRef() { return "host"; }
    static get watchers() {
        return [{
                "propName": "open",
                "methodName": "onOpenChange"
            }, {
                "propName": "htmlTabindex",
                "methodName": "onTabindexChange"
            }];
    }
    static get listeners() {
        return [{
                "name": "keydown",
                "method": "onItemsKeydown",
                "target": undefined,
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=index.js.map
