import { Host, h } from "@stencil/core";
import { KeyboardCode } from "../../beans";
import { containsElement } from "../../utils/utils";
const isZMenuSection = (el) => (el === null || el === void 0 ? void 0 : el.tagName) === "Z-MENU-SECTION";
/**
 * @slot - Menu label
 * @slot header - Header to display as the first entry of the open menu.
 * @slot item - Single entry of the section. Can be slotted multiple times to insert items onto the menu. Set the `data-active` HTML attribute on the element to highlight it (`active` also works but `data-active` is preferable). Use `z-menu-section` for submenus.
 * @cssprop --z-menu-label-color - Color of the label's text.
 */
export class ZMenu {
    get focusableItem() {
        return this.items.find((item) => (isZMenuSection(item) ? item.htmlTabindex === 0 : item.tabIndex === 0));
    }
    toggle() {
        if (!this.hasContent) {
            return;
        }
        this.open = !this.open;
    }
    /**
     * Sets slotted item text as `data-text` attribute value, to let CSS use it through `attr()`.
     * @param ev Slotchange event
     */
    onLabelSlotChange(ev) {
        const labelElement = ev.target.assignedElements()[0];
        labelElement.dataset.text = labelElement === null || labelElement === void 0 ? void 0 : labelElement.textContent;
        this.setLabelA11yAttrs();
    }
    /**
     * Correctly set position of the floating menu in order to prevent overflow.
     * @param live Should run the method on every refresh frame.
     */
    reflow(live = false) {
        if (!this.floating) {
            return;
        }
        if (this.content && this.hasContent) {
            const { style } = this.content;
            const { left } = this.host.getBoundingClientRect();
            const widthPx = getComputedStyle(this.content).width;
            const width = widthPx ? parseFloat(widthPx.replace("px", "")) : 375;
            const safeScrollbarSpace = 30;
            style.left = `${Math.min(window.innerWidth - left - width - safeScrollbarSpace, 0)}px`;
        }
        if (live) {
            this.raf = requestAnimationFrame(this.reflow.bind(this, live));
        }
    }
    /**
     * Check if some content slot is set.
     */
    checkContent() {
        this.hasHeader = !!this.host.querySelectorAll("[slot=header]").length;
        this.hasContent = !!this.host.querySelectorAll("[slot=item]").length || this.hasHeader;
    }
    setItemTabindex(item, tabIndex) {
        if (isZMenuSection(item)) {
            item.htmlTabindex = tabIndex;
        }
        else {
            item.tabIndex = tabIndex;
        }
    }
    /**
     * Set `menuitem` role to all menu items (ZMenuSection items already have it).
     * Set -1 to the tabindex of the items and 0 to the first one.
     */
    setItemsA11yAttrs() {
        this.items.forEach((item, index) => {
            const tabindex = index === 0 ? 0 : -1;
            this.setItemTabindex(item, tabindex);
            if (!isZMenuSection(item)) {
                item.setAttribute("role", "menuitem");
            }
        });
    }
    onItemsChange() {
        this.checkContent();
        this.items = Array.from(this.host.children).filter(({ slot }) => slot === "item");
        this.setItemsA11yAttrs();
        this.items.forEach((item) => {
            if (!isZMenuSection(item)) {
                item.dataset.text = item.textContent;
            }
        });
    }
    /**
     * Move focus and adjust the tabindex value of `receiver` and `current` elements,
     * setting -1 to the `current` and 0 to the `receiver`, then focus the `receiver` element.
     * If the receiver is a ZMenuSection and it's open, focus its first focusable item.
     */
    moveFocus(receiver, current) {
        if (isZMenuSection(receiver)) {
            receiver.setFocus();
        }
        else {
            receiver.tabIndex = 0;
            setTimeout(() => {
                receiver.focus();
            }, 100);
        }
        if (!current) {
            return;
        }
        this.setItemTabindex(current, -1);
    }
    onLabelClick() {
        this.toggle();
        this.setFocus();
    }
    onLabelKeydown(ev) {
        if (ev.key === KeyboardCode.ENTER || ev.key === KeyboardCode.SPACE) {
            ev.preventDefault();
            ev.stopPropagation();
            this.toggle();
            if (this.open) {
                this.moveFocus(this.items[0]);
            }
            return;
        }
        if (!this.verticalContext) {
            return;
        }
        if (ev.key === KeyboardCode.ARROW_RIGHT && !this.open) {
            ev.preventDefault();
            ev.stopPropagation();
            this.open = true;
            this.moveFocus(this.items[0]);
        }
    }
    /** Set tabindex of the label to 0, then focus it. */
    async setFocus() {
        this.htmlTabindex = 0;
        const label = this.hasContent ? this.labelButton : this.host.firstElementChild;
        label.focus();
    }
    /**
     * Focus the last item.
     */
    async focusLastItem() {
        const lastItem = this.items[this.items.length - 1];
        if (isZMenuSection(lastItem) && lastItem.open) {
            lastItem.focusLastItem();
            return;
        }
        this.moveFocus(lastItem);
    }
    onOpenChanged() {
        if (!this.open) {
            cancelAnimationFrame(this.raf);
            this.closed.emit();
            this.items.forEach((item) => {
                if (isZMenuSection(item) && item.open) {
                    item.open = false;
                }
            });
            return;
        }
        this.setItemsA11yAttrs();
        this.opened.emit();
        if (this.floating) {
            this.reflow(true);
        }
    }
    setLabelA11yAttrs() {
        if (this.hasContent && this.labelButton) {
            this.labelButton.tabIndex = this.htmlTabindex;
            return;
        }
        const slottedLabel = this.host.firstElementChild;
        slottedLabel.role = "menuitem";
        slottedLabel.tabIndex = this.htmlTabindex;
    }
    /**
     * Set tabindex to -1 to all siblings of the clicked item.
     */
    onItemClick(ev) {
        const clickedItem = this.items.find((item) => containsElement(item, ev.target));
        if (clickedItem) {
            this.items.forEach((item) => {
                if (item === clickedItem) {
                    return;
                }
                if (isZMenuSection(item)) {
                    item.htmlTabindex = -1;
                }
                else {
                    item.tabIndex = -1;
                }
            });
        }
    }
    onKeyDown(ev) {
        var _a;
        if (!this.hasContent) {
            return;
        }
        switch (ev.key) {
            case KeyboardCode.ESC:
                if (!this.open) {
                    break;
                }
                ev.stopPropagation();
                ev.preventDefault();
                this.open = false;
                this.setFocus();
                break;
            case KeyboardCode.ARROW_DOWN: {
                if (document.activeElement === this.host) {
                    if (this.verticalContext && !this.open) {
                        break;
                    }
                    ev.stopPropagation();
                    ev.preventDefault();
                    if (!this.open) {
                        this.open = true;
                    }
                    this.moveFocus(this.items[0]);
                    break;
                }
                const currentIndex = this.items.indexOf(this.focusableItem);
                if (this.verticalContext && currentIndex === this.items.length - 1) {
                    // navigation is going to leave this menu. restore tabindex to the label and let the parent handle it
                    this.setItemTabindex(this.items[currentIndex], -1);
                    this.htmlTabindex = 0;
                    break;
                }
                ev.stopPropagation();
                ev.preventDefault();
                const receiver = this.items[currentIndex + 1];
                // if the last item is already focused, navigate to the first one
                this.moveFocus(receiver !== null && receiver !== void 0 ? receiver : this.items[0], this.focusableItem);
                break;
            }
            case KeyboardCode.ARROW_UP: {
                if (document.activeElement === this.host) {
                    if (this.verticalContext) {
                        break;
                    }
                    ev.stopPropagation();
                    ev.preventDefault();
                    // open the menu and focus the last item
                    if (!this.open) {
                        this.open = true;
                    }
                    this.moveFocus(this.items[this.items.length - 1], this.focusableItem);
                    break;
                }
                ev.stopPropagation();
                ev.preventDefault();
                const currentIndex = this.items.indexOf(this.focusableItem);
                if (currentIndex === 0 && this.verticalContext) {
                    this.setItemTabindex(this.focusableItem, -1);
                    this.setFocus();
                    break;
                }
                const receiver = (_a = this.items[currentIndex - 1]) !== null && _a !== void 0 ? _a : this.items[this.items.length - 1];
                // if the receiver is a ZMenuSection and it's open, focus its last item
                if (isZMenuSection(receiver) && receiver.open) {
                    this.setItemTabindex(this.focusableItem, -1);
                    receiver.focusLastItem();
                    break;
                }
                this.moveFocus(receiver, this.focusableItem);
                break;
            }
            case KeyboardCode.ARROW_LEFT:
                if (!this.open || !this.verticalContext) {
                    break;
                }
                // close the menu and focus the label
                ev.preventDefault();
                ev.stopPropagation();
                this.open = false;
                this.setFocus();
        }
    }
    constructor() {
        this.items = [];
        this.active = undefined;
        this.floating = true;
        this.open = false;
        this.verticalContext = false;
        this.htmlTabindex = -1;
        this.hasHeader = undefined;
        this.hasContent = undefined;
        this.toggle = this.toggle.bind(this);
        this.checkContent = this.checkContent.bind(this);
        this.onLabelSlotChange = this.onLabelSlotChange.bind(this);
        this.onLabelClick = this.onLabelClick.bind(this);
        this.onItemsChange = this.onItemsChange.bind(this);
        this.onLabelKeydown = this.onLabelKeydown.bind(this);
    }
    componentWillLoad() {
        this.setLabelA11yAttrs();
        this.onItemsChange();
    }
    render() {
        if (!this.hasContent) {
            return (h("div", { class: "menu-label" }, h("slot", { onSlotchange: this.onLabelSlotChange })));
        }
        return (h(Host, null, h("button", { ref: (el) => (this.labelButton = el), class: "menu-label", "aria-expanded": `${!!this.open}`, "aria-haspopup": `${this.hasContent}`, "aria-label": this.open ? "Chiudi menù" : "Apri menù", role: "menuitem", tabIndex: this.htmlTabindex, onClick: this.onLabelClick, onKeyDown: this.onLabelKeydown }, h("slot", { onSlotchange: this.onLabelSlotChange }), h("z-icon", { name: this.open ? "chevron-up" : "chevron-down" })), h("div", { class: "content", ref: (el) => (this.content = el) }, this.hasHeader && (h("header", { class: "header" }, h("slot", { name: "header", onSlotchange: this.checkContent }))), h("div", { class: "items", role: "menu" }, h("slot", { name: "item", onSlotchange: this.onItemsChange })))));
    }
    static get is() { return "z-menu"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["styles.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["styles.css"]
        };
    }
    static get properties() {
        return {
            "active": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Flag to set the active status of the menu."
                },
                "attribute": "active",
                "reflect": true
            },
            "floating": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Flag to set the display mode of the list.\nIf true, the list will be absolutely positioned under the menu label,\nstacked beneath it otherwise."
                },
                "attribute": "floating",
                "reflect": true,
                "defaultValue": "true"
            },
            "open": {
                "type": "boolean",
                "mutable": true,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The opening state of the menu."
                },
                "attribute": "open",
                "reflect": true,
                "defaultValue": "false"
            },
            "verticalContext": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Tells the component that it's placed in a vertical context with other `ZMenu`s (usually in the ZAppHeader's offcanvas).\nA small border is placed under it as a separator from other elements."
                },
                "attribute": "vertical-context",
                "reflect": true,
                "defaultValue": "false"
            },
            "htmlTabindex": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Tabindex value to set on the menu label.\nUseful to manage keyboard navigation focus with roving tabindex handled by this component's parent (usually ZAppHeader)."
                },
                "attribute": "html-tabindex",
                "reflect": false,
                "defaultValue": "-1"
            }
        };
    }
    static get states() {
        return {
            "hasHeader": {},
            "hasContent": {}
        };
    }
    static get events() {
        return [{
                "method": "opened",
                "name": "opened",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "The menu has been opened."
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }, {
                "method": "closed",
                "name": "closed",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "The menu has been closed."
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
    static get methods() {
        return {
            "setFocus": {
                "complexType": {
                    "signature": "() => Promise<void>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        },
                        "HTMLElement": {
                            "location": "global",
                            "id": "global::HTMLElement"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "Set tabindex of the label to 0, then focus it.",
                    "tags": []
                }
            },
            "focusLastItem": {
                "complexType": {
                    "signature": "() => Promise<void>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "Focus the last item.",
                    "tags": []
                }
            }
        };
    }
    static get elementRef() { return "host"; }
    static get watchers() {
        return [{
                "propName": "open",
                "methodName": "onOpenChanged"
            }, {
                "propName": "htmlTabindex",
                "methodName": "setLabelA11yAttrs"
            }];
    }
    static get listeners() {
        return [{
                "name": "click",
                "method": "onItemClick",
                "target": "document",
                "capture": false,
                "passive": false
            }, {
                "name": "keydown",
                "method": "onKeyDown",
                "target": undefined,
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=index.js.map
