import { h, Host } from "@stencil/core";
import { OffCanvasVariant, TransitionDirection } from "../../beans";
/**
 * @slot canvasContent - Slot for the offcanvas content.
 * @cssprop --z-offcanvas--top-space - Top offset of the offcanvas, for `overlay` variant. Useful when there is some fixed element above the offcanvas. Default: `0`.
 * @cssprop --z-offcanvas--container-width - Width of the offcanvas for `left` and `right` direction. Default: `375px`.
 * @cssprop --z-offcanvas--container-height - Height of the offcanvas for `up` direction. Default: `90%`.
 */
export class ZOffcanvas {
    constructor() {
        this.variant = OffCanvasVariant.PUSHCONTENT;
        this.open = false;
        this.transitiondirection = TransitionDirection.LEFT;
        this.skipLoadAnimation = false;
        this.skipAnimation = false;
    }
    /**
     * Stop the event default behavior and propagation when the offcanvas is closed.
     */
    stopEvent(event) {
        if (this.open) {
            return;
        }
        event.preventDefault();
        event.stopPropagation();
    }
    /**
     * Hide the body overflow when the offcanvas is open.
     */
    handlePageOverflow() {
        const overflow = this.variant === OffCanvasVariant.OVERLAY || this.transitiondirection === TransitionDirection.UP
            ? "overflow-y"
            : "overflow-x";
        document.body.style[overflow] = this.open ? "hidden" : "";
    }
    onOpenChanged() {
        var _a;
        this.canvasOpenStatusChanged.emit(this.open);
        if (!this.open) {
            // when the offcanvas is closing, wait for the transitionend event to end before handling the body overflow
            const listenerCallback = () => {
                this.handlePageOverflow();
                this.canvasContainer.removeEventListener("transitionend", listenerCallback);
            };
            (_a = this.canvasContainer) === null || _a === void 0 ? void 0 : _a.addEventListener("transitionend", listenerCallback);
        }
        else {
            this.handlePageOverflow();
        }
    }
    componentWillLoad() {
        if (this.open) {
            this.skipAnimation = true;
        }
    }
    componentDidLoad() {
        this.handlePageOverflow();
    }
    componentDidRender() {
        this.skipAnimation = false;
    }
    connectedCallback() {
        this.handlePageOverflow();
    }
    disconnectedCallback() {
        this.open = false;
    }
    render() {
        return (h(Host, { key: '299c69d8788f7cf2e12b6f6eb151c3b7c5e4e271', "skip-animation": this.skipAnimation, "aria-hidden": !this.open ? "true" : null }, h("div", { key: '2620f2f0ef0741cbe6e5d63f7ed38299fb253852', class: "canvas-container", role: "presentation", ref: (el) => (this.canvasContainer = el) }, h("div", { key: '1df851d5c4d1fddb24a62c5e3d92a7181bd36d05', class: "canvas-content z-scrollbar", role: "presentation" }, h("slot", { key: '89303666352d55c9a92a0b4cecc9bea8d42125f7', name: "canvasContent" }))), (this.variant == OffCanvasVariant.OVERLAY || this.transitiondirection === TransitionDirection.UP) && (h("div", { key: '197e3eb34f5981c52a553875157a3a02993f7684', class: "canvas-background", onClick: () => (this.open = false) }))));
    }
    static get is() { return "z-offcanvas"; }
    static get encapsulation() { return "scoped"; }
    static get originalStyleUrls() {
        return {
            "$": ["styles.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["styles.css"]
        };
    }
    static get properties() {
        return {
            "variant": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "OffCanvasVariant",
                    "resolved": "OffCanvasVariant.OVERLAY | OffCanvasVariant.PUSHCONTENT",
                    "references": {
                        "OffCanvasVariant": {
                            "location": "import",
                            "path": "../../beans",
                            "id": "src/beans/index.tsx::OffCanvasVariant"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Offcanvas variant.\n- `overlay`: The offcanvas covers the page content putting an overlay.\n- `pushcontent`: The offcanvas isn't absolutely positioned and \"pushes\" the page content.\n> NB: `pushcontent` variant may need some extra style tuning of the context around the component to work properly.\nAlso, the `pushcontent` variant doesn't have the open/close transition."
                },
                "attribute": "variant",
                "reflect": true,
                "defaultValue": "OffCanvasVariant.PUSHCONTENT"
            },
            "open": {
                "type": "boolean",
                "mutable": true,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Whether the offcanvas is open. Default: false"
                },
                "attribute": "open",
                "reflect": true,
                "defaultValue": "false"
            },
            "transitiondirection": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "TransitionDirection",
                    "resolved": "TransitionDirection.LEFT | TransitionDirection.RIGHT | TransitionDirection.UP",
                    "references": {
                        "TransitionDirection": {
                            "location": "import",
                            "path": "../../beans",
                            "id": "src/beans/index.tsx::TransitionDirection"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Open the content transitioning to a specified direction.\nThe `up` direction always show the `overlay`, also when the variant is `pushcontent`."
                },
                "attribute": "transitiondirection",
                "reflect": true,
                "defaultValue": "TransitionDirection.LEFT"
            },
            "skipLoadAnimation": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "deprecated",
                            "text": "This prop is not needed anymore, the component will automatically skip the transition\nwhen it starts with `open` set to `true`. Only exists for Typescript backward compatibility."
                        }],
                    "text": "Skip the opening transition when the offcanvas is already open at the first render."
                },
                "attribute": "skip-load-animation",
                "reflect": false,
                "defaultValue": "false"
            }
        };
    }
    static get states() {
        return {
            "skipAnimation": {}
        };
    }
    static get events() {
        return [{
                "method": "canvasOpenStatusChanged",
                "name": "canvasOpenStatusChanged",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "emitted when `open` prop changes"
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
    static get watchers() {
        return [{
                "propName": "variant",
                "methodName": "handlePageOverflow"
            }, {
                "propName": "open",
                "methodName": "onOpenChanged"
            }];
    }
    static get listeners() {
        return [{
                "name": "click",
                "method": "stopEvent",
                "target": undefined,
                "capture": true,
                "passive": false
            }, {
                "name": "focusin",
                "method": "stopEvent",
                "target": undefined,
                "capture": true,
                "passive": false
            }];
    }
}
//# sourceMappingURL=index.js.map
